package dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import dto.Gender;
import dto.Person;

public class PersonBMICalculatorDAO {

	private static PersonBMICalculatorDAO instance = new PersonBMICalculatorDAO();
	public static PersonBMICalculatorDAO getInstance(){
		return instance;
	}
	
	private PersonBMICalculatorDAO() {
	}
	
	// insert
	public int insert(Connection conn, Person person) throws SQLException {
		PreparedStatement pstmt = null;
		try {
			String sql = "insert into person values (?,?,?,?,?,?,?,?)";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, person.getName());
			pstmt.setInt(2, person.getAge());
			pstmt.setString(3, person.getGender().toString());
			pstmt.setDouble(4, person.getHeight());
			pstmt.setDouble(5, person.getWeight());
			pstmt.setDouble(6, person.getNeck());
			pstmt.setDouble(7, person.getWaist());
			pstmt.setDouble(8, person.getHip());
			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}

	// delete
	public int delete(Connection conn, String name) throws SQLException {
		PreparedStatement pstmt = null;
		try {
			String sql = "delete from person where name=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, name);
			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}

	// edit
	public int edit(Connection conn, Person person) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "update person set age=?, gender=?, height=?, weight=?, neck=?, waist=?, hip=? where name=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, person.getAge());
			pstmt.setString(2, person.getGender().toString());
			pstmt.setDouble(3, person.getHeight());
			pstmt.setDouble(4, person.getWeight());
			pstmt.setDouble(5, person.getNeck());
			pstmt.setDouble(6, person.getWaist());
			pstmt.setDouble(7, person.getHip());
			pstmt.setString(8, person.getName());
			return pstmt.executeUpdate();
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	// select(find/get)
	public Person select(Connection conn, String name) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from person where name=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, name);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return createFromResultSet(rs);
			}
			else {
				return null;
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	public Person createFromResultSet(ResultSet rs) throws SQLException {
		String name = rs.getString("name");
		int age = Integer.parseInt(rs.getString("age"));
		Gender gender = Gender.nameOf(rs.getString("gender"));
		Double height = rs.getDouble("height");
		Double weight = rs.getDouble("weight");
		Double neck = rs.getDouble("neck");
		Double waist = rs.getDouble("waist");
		Double hip = rs.getDouble("hip");
		Person person = new Person(name, age, gender, height, weight, neck, waist, hip);
		return person;
	}
	
	// selectList
	public List<Person> selectList(Connection conn) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from person";
			pstmt = conn.prepareStatement(sql);
			rs = pstmt.executeQuery();
			List<Person> personList = new ArrayList<>();
			while (rs.next()) {
				personList.add(createFromResultSet(rs));
			}
			return personList;
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
}
